import Delta from "quill-delta-es";
import { type BlockyElement, type AttributesObject, type BlockyNode, type BlockElement } from "./tree";
import { type Operation, type OperationMessage } from "./operations";
import { NodeLocation } from "./location";
import type { CursorState } from "./cursor";
export declare enum ChangesetRecordOption {
    Undo = 0,
    Redo = 1,
    None = 2
}
export interface ChangesetApplyOptions {
    updateView: boolean;
    ignoreCursor: boolean;
    record: ChangesetRecordOption;
    refreshCursor: boolean;
}
export interface ChangesetStateLogger {
    userId: string;
    get appliedVersion(): number;
    get cursorState(): CursorState | null;
    getLocationOfNode(node: BlockyNode): NodeLocation;
    apply(changeset: FinalizedChangeset): void;
}
/**
 * Changeset is a collection of changes which can be
 * applied to the document's state.
 *
 * Each changeset has it's own ID.
 * So it can be applied repeatedly.
 */
export declare class Changeset {
    readonly state: ChangesetStateLogger;
    version: number;
    operations: Operation[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    constructor(state: ChangesetStateLogger);
    updateAttributes(node: BlockyElement, attributes: AttributesObject): Changeset;
    setCursorState(cursorState: CursorState | null): Changeset;
    appendChild(node: BlockyElement, child: BlockyNode): Changeset;
    removeChild(parent: BlockyElement, child: BlockyNode): Changeset;
    removeNode(node: BlockyNode): Changeset;
    deleteChildrenAt(parent: BlockyElement, index: number, count: number): Changeset;
    insertChildrenAfter(parent: BlockyElement, children: BlockyNode[], after?: BlockyNode | null): Changeset;
    insertChildrenAt(parent: BlockyElement, index: number, children: BlockyNode[]): Changeset;
    textEdit(node: BlockElement, propName: string, delta: () => Delta): Changeset;
    textConcat(node: BlockElement, propName: string, delta: () => Delta): Changeset;
    /**
     * This method will transform path.
     * If you don't want to transform path, call [pushWillMerge].
     */
    push(operation: Operation): void;
    pushWillMerge(operation: Operation): void;
    apply(options?: Partial<ChangesetApplyOptions>): void;
    finalize(options?: Partial<ChangesetApplyOptions>): FinalizedChangeset;
    /**
     * @param that Can be either a [Changeset] or a [FinalizedChangeset]
     */
    append(that: Changeset | FinalizedChangeset): void;
}
export interface FinalizedChangeset {
    userId: string;
    version: number;
    operations: Operation[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    options: ChangesetApplyOptions;
}
export interface ChangesetMessage {
    userId: string;
    version: number;
    operations: OperationMessage[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    options: ChangesetApplyOptions;
}
export declare function changesetToMessage(changeset: FinalizedChangeset): ChangesetMessage;
export declare function changesetFromMessage(msg: ChangesetMessage): FinalizedChangeset;
